﻿using System;
using Eco.Persistence;
using Microsoft.SqlServer.Types;
using Eco.Persistence.Default;
using Eco.Persistence.Configuration;
using System.Data;
using System.Data.SqlTypes;
using System.Data.SqlClient;

namespace Eco.Persistence.SqlServer
{
    public sealed class SqlGeometryAsGeometry : AbstractSingleColumnAttribute, ISingleColumnAttributemapping
    {
        public System.Type ValueType()
        {
            return typeof(SqlGeometry);
        }

        public String ColumnType(int length)
        {
            return "GEOMETRY"; // do not localize
        }

        ///<exception cref="ArgumentNullException">Thrown if <paramref name="sqlDatabaseConfig"/> is null</exception>
        public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
        {
            if (String.IsNullOrEmpty(modeledValue))
                return "'POINT(0 0)'";
            else
                return modeledValue;
        }

        public void ValueToParameter(System.Object value, IDataParameter parameter)
        {
            if (parameter == null)
                throw new ArgumentNullException("parameter");
            EnsureType(value, typeof(SqlGeometry));
            (parameter as SqlParameter).UdtTypeName = "Geometry";
            //parameter.DbType = DbType.DateTime;
            if (value == null)
                parameter.Value = DBNull.Value;
            else
                parameter.Value = value;
        }

        public void StringToParameter(string value, IDataParameter parameter)
        {
            ValueToParameter(SqlGeometry.STGeomFromText(new SqlChars(new SqlString(value)), 0), parameter);
        }

        public override System.Object ColumnToValue(System.Object columnValue)
        {
            EnsureType(columnValue, typeof(SqlGeometry));
            if (DBNull.Value.Equals(columnValue) || columnValue == null)
                return null;
            else
                return columnValue;
        }
    }
    public sealed class SqlGeographyAsGeography : AbstractSingleColumnAttribute, ISingleColumnAttributemapping
    {
        public System.Type ValueType()
        {
            return typeof(SqlGeography);
        }

        public String ColumnType(int length)
        {
            return "GEOGRAPHY"; // do not localize
        }

        ///<exception cref="ArgumentNullException">Thrown if <paramref name="sqlDatabaseConfig"/> is null</exception>
        public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
        {
            if (String.IsNullOrEmpty(modeledValue))
                return "'POINT(0 0)'";
            else
                return modeledValue;
        }

        public void ValueToParameter(System.Object value, IDataParameter parameter)
        {
            if (parameter == null)
                throw new ArgumentNullException("parameter");
            EnsureType(value, typeof(SqlGeography));
            (parameter as SqlParameter).UdtTypeName = "Geography";
            if (value == null)
                parameter.Value = DBNull.Value;
            else
                parameter.Value = value;
        }

        public void StringToParameter(string value, IDataParameter parameter)
        {
            ValueToParameter(SqlGeography.STPointFromText(new SqlChars(new SqlString(value)), 0), parameter);
        }

        public override System.Object ColumnToValue(System.Object columnValue)
        {
            EnsureType(columnValue, typeof(SqlGeography));
            if (DBNull.Value.Equals(columnValue) || columnValue == null)
                return null;
            else
                return columnValue;
        }
    }
}
