using System;
using System.Data.Common;
using System.Drawing;
using System.Collections.Specialized;
using System.ComponentModel;
using System.ComponentModel.Design;
using Microsoft.Data.SqlClient;

using Eco.Interfaces;
using Eco.Persistence.Connection;
using Eco.Persistence.Configuration;
using Eco.Globalization;

namespace Eco.Persistence
{
	///<summary>
	///Persistencemapper using Microsoft SqlConnection connections.
	///</summary>
	[ToolboxBitmap(typeof(PersistenceMapperSqlServer), "Eco.Persistence.PersistenceMapperSqlServer.bmp")]
  [Designer("Eco.Persistence.Design.SqlServerPersistenceHandleDesigner, Eco.Persistence.SqlServer.Design" + VersionInfo.AssemblySuffixForDesignerQualification, typeof(IDesigner))]
	[ToolboxItem(true)]
	public class PersistenceMapperSqlServer: PersistenceMapperAdo2
	{

	        [Editor("Microsoft.VSDesigner.Data.SQL.Design.SqlConnectionStringEditor, Microsoft.VSDesigner, Version=8.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
	        public override string ConnectionString
	        {
	            get { return base.ConnectionString; }
	            set { base.ConnectionString = value; }
	        }

		protected override Type ConnectionType() { return typeof(SqlConnection); }

		[LocalizableCategory(typeof(PersistenceStringRes), "sCategoryPersistence")]
		[LocalizableDescription(typeof(PersistenceStringRes), "sPropertyConnection")]
		[DefaultValue(null)]
    public DbConnection Connection
		{
			get { return DbConnection; }
			set { DbConnection = value; }
		}

    private bool _isSQLCe = false;
    /// <summary>
    /// SQLCe requires some special treatment like timeout of 0 and use of nvarchar instead of varchar
    /// </summary>
    [DefaultValue(false)]
    public bool IsSQLCe
    {
      get{ return _isSQLCe; }
      set 
      { 
        _isSQLCe = value;
        if (_isSQLCe)
        {
          CommandTimeout = 0;

        }
      }
    }

		protected override IDatabase InternalCreateIDatabase(DbConnection connection, SqlDatabaseConfig config)
		{
			return new SqlIDatabaseImpl(connection, config);
		}

		static PersistenceMapperSqlServer()
		{
			RegisterAttributeMappers();
		}

		private static bool attributeMappersRegistered = false;
		public static void RegisterAttributeMappers()
		{
			if (!attributeMappersRegistered)
			{
				// MsAccess
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DataBaseEngine.AccessSqlClient, StringLiterals.SystemByteArr, typeof(Eco.Persistence.SqlServer.SqlClient.ByteArrayAsImage))); // do not localize
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DataBaseEngine.AccessSqlClient, "StringAsText", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsText))); // do not localize
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DataBaseEngine.AccessSqlClient, "StringAsNText", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNText))); // do not localize

				// SqlServer
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlClient, StringLiterals.SystemByteArr, typeof(Eco.Persistence.SqlServer.SqlClient.ByteArrayAsImage))); // do not localize
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlClient, "StringAsText", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsText))); // do not localize
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlClient, "StringAsNText", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNText))); // do not localize
				DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlClient, "EcoOrMapping", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNText))); // do not localize
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlClient, "StringAsNVarCharMax", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNVarCharMax))); // do not localize

        // SqlCe
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlCe, StringLiterals.SystemByteArr, typeof(Eco.Persistence.SqlServer.SqlClient.ByteArrayAsImage))); // do not localize
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlCe, "StringAsText", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNText))); // SQLCE does not allow VARCHAR only nVARCHAR // do not localize
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlCe, "StringAsNText", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNText))); // do not localize
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlCe, "EcoOrMapping", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNText))); // do not localize
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlCe, "StringAsNVarCharMax", typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNVarChar510))); // do not localize
        DatabaseEngineInitializer.AttributeMapperList.Add(new MapperDefinition(DatabaseEngineInitializer.sqlServerSqlCe, StringLiterals.SystemString, typeof(Eco.Persistence.SqlServer.SqlClient.StringAsNVarChar510))); // do not localize

      }
			attributeMappersRegistered = true;
		}

	}
}
