using System;
using System.Data;
using System.Data.SqlTypes;
using Microsoft.Data.SqlClient;

using Eco.Persistence;
using Eco.Persistence.Default;
using Eco.Persistence.Configuration;

namespace Eco.Persistence.SqlServer.SqlClient
{

  public class SqlCeHelper
  {
    private static void ConsiderSQLCESetSqlDbTypeOnParam(IDataParameter parameter, SqlDbType dbtype)
    {
      // keep this in seperate method since JIT will fail if System.Data.SqlServerCe is not found
      if (parameter is System.Data.SqlServerCe.SqlCeParameter)
        ((System.Data.SqlServerCe.SqlCeParameter)parameter).SqlDbType = dbtype;

    }
    public static void SetSqlDbTypeOnParam(IDataParameter parameter, SqlDbType dbtype)
    {
      if (parameter is SqlParameter)
        ((SqlParameter)parameter).SqlDbType = dbtype;
      else 
        ConsiderSQLCESetSqlDbTypeOnParam(parameter, dbtype);
    }
  }

  
  ///<summary>
	///Mapper between a ByteArray value and its persistent representation as IMAGE.
	///</summary>
    public sealed class ByteArrayAsImage : AbstractSingleColumnAttribute, ISingleColumnAttributemapping, IDefaultReverseMapping
	{
		public System.Type ValueType()
		{
			return typeof(System.Byte[]);
		}

		public String ColumnType(int length)
		{
			return "IMAGE"; // do not localize
		}

		public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
		{
			return modeledValue;
		}

		public void ValueToParameter(System.Object value, IDataParameter parameter)
		{
			parameter.DbType = DbType.Binary;
      SqlCeHelper.SetSqlDbTypeOnParam(parameter, SqlDbType.Image);
			if (value == null)
				parameter.Value = DBNull.Value;
			else
				parameter.Value = value;
		}


		///<exception cref="InvalidOperationException">Thrown always.</exception>
		public void StringToParameter(string value, IDataParameter parameter)
		{
			throw new InvalidOperationException(PersistenceStringRes.sCannotConvertImageToString);
		}

		public override System.Object ColumnToValue(System.Object columnValue)
		{
			if (DBNull.Value.Equals(columnValue) || columnValue == null)
				return null;
			else
				return columnValue;
		}

		public override bool IsEqual(System.Object value, System.Object columnValue)
		{
			return MappingUtils.ByteArraysAreEqual(value, ColumnToValue(columnValue));
		}

	}

    ///<summary>
    ///Mapper between a ByteArray value and its persistent representation as VARBINARY(MAX).
    ///</summary>
    public sealed class ByteArrayAsVarbinaryMax : AbstractSingleColumnAttribute, ISingleColumnAttributemapping, IDefaultReverseMapping
    {
      public System.Type ValueType()
      {
        return typeof(System.Byte[]);
      }

      public String ColumnType(int length)
      {
        return "VARBINARY(MAX)"; // do not localize
      }

      public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
      {
        return modeledValue;
      }

      public void ValueToParameter(System.Object value, IDataParameter parameter)
      {
        parameter.DbType = DbType.Binary;
        SqlCeHelper.SetSqlDbTypeOnParam(parameter, SqlDbType.VarBinary);
        if (value == null)
          parameter.Value = DBNull.Value;
        else
          parameter.Value = value;
      }


      ///<exception cref="InvalidOperationException">Thrown always.</exception>
      public void StringToParameter(string value, IDataParameter parameter)
      {
        throw new InvalidOperationException(PersistenceStringRes.sCannotConvertImageToString);
      }

      public override System.Object ColumnToValue(System.Object columnValue)
      {
        if (DBNull.Value.Equals(columnValue) || columnValue == null)
          return null;
        else
          return columnValue;
      }

      public override bool IsEqual(System.Object value, System.Object columnValue)
      {
        return MappingUtils.ByteArraysAreEqual(value, ColumnToValue(columnValue));
      }

    }

    public sealed class StringAsText : AbstractSingleColumnAttribute, ISingleColumnAttributemapping, IDefaultReverseMapping, ICustomOrderByMapping
	{
		public System.Type ValueType()
		{
			return typeof(System.String);
		}
		public String ColumnType(int length)
		{
			return "TEXT"; // do not localize
		}

		public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
		{
			return modeledValue;
		}

		public void ValueToParameter(System.Object value, IDataParameter parameter)
		{
			EnsureType(value, typeof(System.String));
      SqlCeHelper.SetSqlDbTypeOnParam(parameter, SqlDbType.Text);
      ((SqlParameter)parameter).SqlDbType = SqlDbType.Text;
			if (value == null)
				parameter.Value = DBNull.Value;
			else
				parameter.Value = value;
		}

		public void StringToParameter(string value, IDataParameter parameter)
		{
			ValueToParameter(value, parameter);
		}

		public override System.Object ColumnToValue(System.Object columnValue)
		{
			EnsureType(columnValue, typeof(System.String));
			if (DBNull.Value.Equals(columnValue) || columnValue == null)
				return null;
			else
				return columnValue;
		}


    #region ICustomOrderByMapping Members

    public string GetOrderableColumn(string column)
    {
      return string.Format("CAST({0} as NVARCHAR(4000))", column);
    }

    #endregion
  }

    public sealed class StringAsNVarChar510 : AbstractSingleColumnAttribute, ISingleColumnAttributemapping, IDefaultReverseMapping
    {
      public System.Type ValueType()
      {
        return typeof(System.String);
      }
      public String ColumnType(int length)
      {
        return "NVARCHAR(510)"; // 510 is max in sqlce do not localize
      }

      public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
      {
        return modeledValue;
      }

      public void ValueToParameter(System.Object value, IDataParameter parameter)
      {
        EnsureType(value, typeof(System.String));
        SqlCeHelper.SetSqlDbTypeOnParam(parameter, SqlDbType.NVarChar);
        if (value == null)
          parameter.Value = DBNull.Value;
        else
          parameter.Value = value;
      }

      public void StringToParameter(string value, IDataParameter parameter)
      {
        ValueToParameter(value, parameter);
      }

      public override System.Object ColumnToValue(System.Object columnValue)
      {
        EnsureType(columnValue, typeof(System.String));
        if (DBNull.Value.Equals(columnValue) || columnValue == null)
          return null;
        else
          return columnValue;
      }

    }


    public sealed class StringAsNVarCharMax : AbstractSingleColumnAttribute, ISingleColumnAttributemapping, IDefaultReverseMapping
    {
        public System.Type ValueType()
        {
            return typeof(System.String);
        }
        public String ColumnType(int length)
        {
            return "NVARCHAR(MAX)"; // do not localize
        }

        public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
        {
            return modeledValue;
        }

        public void ValueToParameter(System.Object value, IDataParameter parameter)
        {
            EnsureType(value, typeof(System.String));
            SqlCeHelper.SetSqlDbTypeOnParam(parameter, SqlDbType.NVarChar); 
            if (value == null)
                parameter.Value = DBNull.Value;
            else
                parameter.Value = value;
        }

        public void StringToParameter(string value, IDataParameter parameter)
        {
            ValueToParameter(value, parameter);
        }

        public override System.Object ColumnToValue(System.Object columnValue)
        {
            EnsureType(columnValue, typeof(System.String));
            if (DBNull.Value.Equals(columnValue) || columnValue == null)
                return null;
            else
                return columnValue;
        }

    }

    public sealed class StringAsNText : AbstractSingleColumnAttribute, ISingleColumnAttributemapping, IDefaultReverseMapping
	{
		public System.Type ValueType()
		{
			return typeof(System.String);
		}
		public String ColumnType(int length)
		{
			return "NTEXT"; // do not localize
		}
		public String DefaultDbValue(string modeledValue, SqlDatabaseConfig sqlDatabaseConfig)
		{
			return modeledValue;
		}

		public void ValueToParameter(System.Object value, IDataParameter parameter)
		{
			EnsureType(value, typeof(System.String));
      SqlCeHelper.SetSqlDbTypeOnParam(parameter, SqlDbType.NText);
      if (value == null)
				parameter.Value = DBNull.Value;
			else
				parameter.Value = value;
		}

		public void StringToParameter(string value, IDataParameter parameter)
		{
			ValueToParameter(value, parameter);
		}

		public override System.Object ColumnToValue(System.Object columnValue)
		{
			EnsureType(columnValue, typeof(System.String));
			if (DBNull.Value.Equals(columnValue) || columnValue == null)
				return null;
			else
				return columnValue;
		}

	}

}

